@@ -309,21 +309,24 @@ class AbstractProblem(ABC):
         """
         pop = []
         for solution in population:
+            try:
                 res = self.simplify(solution)
                 pop.extend(res)
+            except Exception:
+                pop.append(solution)
         return filter_duplicates(pop)
 
 
 class AbstractKOProblem(AbstractProblem):
-    """
-    Base class for Knockout optimization problems.
-
-    :param model: The constraint metabolic model.
-    :param list fevaluation: A list of callable EvaluationFunctions.
-
-    """
 
     def __init__(self, model, fevaluation=None, **kwargs):
+        """
+        Base class for Knockout optimization problems.
+
+        :param model: The constraint metabolic model.
+        :param list fevaluation: A list of callable EvaluationFunctions.
+
+        """
         super(AbstractKOProblem, self).__init__(
             model, fevaluation=fevaluation, **kwargs)
         self.strategy = Strategy.KO
@@ -368,11 +371,14 @@ class AbstractKOProblem(AbstractProblem):
         """
         Generates a solution, a random int set with length in range min_solution_size to max_solution_size.
         """
-        solution = set()
-        solution_size = random.uniform(
-            self.candidate_min_size, self.candidate_max_size)
-        while len(solution) < solution_size:
-            solution.add(random.randint(0, len(self.target_list) - 1))
+
+        if self.candidate_min_size == self.candidate_max_size:
+            solution_size = self.candidate_min_size
+        else:
+            solution_size = random.randint(
+                self.candidate_min_size, self.candidate_max_size)
+
+        solution = set(random.sample(range(len(self.target_list)), solution_size))
         return solution
 
 
@@ -390,6 +396,7 @@ class AbstractOUProblem(AbstractProblem):
         :param dic reference: Dictionary of flux values to be used in the over/under expression values computation.
         :param list levels: Over/under expression levels (Default EAConstants.LEVELS).
         :param boolean twostep: If deletions should be applied before identifiying reference flux values.
+        :param dict partial_solution: A partial solution to be appended to any other solution
         """
         super(AbstractOUProblem, self).__init__(
             model, fevaluation=fevaluation, **kwargs)
@@ -397,10 +404,11 @@ class AbstractOUProblem(AbstractProblem):
         self.levels = kwargs.get('levels', EAConstants.LEVELS)
         self._reference = kwargs.get('reference', None)
         self.twostep = kwargs.get('twostep', True)
+        self._partial_solution = kwargs.get('partial_solution', dict())
 
     def decode(self, candidate):
         """The decoder function for the problem. Needs to be implemented by extending classes."""
-        decoded = dict()
+        decoded = self._partial_solution.copy()
         for idx, lv_idx in candidate:
             try:
                 rxn = self.target_list[idx]
@@ -451,18 +459,16 @@ class AbstractOUProblem(AbstractProblem):
         :param dict args: A dictionary of additional parameters.
         :returns: A new solution.
         """
-        solution = set()
-        solution_size = random.uniform(
-            self.candidate_min_size, self.candidate_max_size)
-        idxs = []
-        while len(solution) < solution_size:
-            idx = random.randint(0, len(self.target_list) - 1)
-            lv = random.randint(0, len(self.levels) - 1)
-            # idx = self.target_list.index(random.choice(self.target_list))
-            # lv = self.levels.index(random.choice(self.levels))
-            if idx not in idxs:
-                solution.add((idx, lv))
-                idxs.append(idx)
+
+        if self.candidate_min_size == self.candidate_max_size:
+            solution_size = self.candidate_min_size
+        else:
+            solution_size = random.randint(
+                self.candidate_min_size, self.candidate_max_size)
+
+        keys = random.sample(range(len(self.target_list)), solution_size)
+        values = random.choices(range(len(self.levels)), k=solution_size)
+        solution = set(zip(keys, values))
         return solution
 
     @property
