@@ -26,9 +26,7 @@ class CobraModelContainer(ModelContainer):
 
     """
 
-    def __init__(self, model: Model):
-        if not isinstance(model, Model):
-            raise ValueError("The model is not an instance of cobrapy Model")
+    def __init__(self, model: Model = None):
         self.model = model
 
     @property
@@ -95,18 +93,27 @@ class CobraModelContainer(ModelContainer):
         else:
             return None
 
-    def get_substrates(self, rxn_id):
-        reaction = self.model.reactions.get_by_id(rxn_id)
-        return {k.id: v for k, v in reaction.metabolites.items() if v < 0}
-
-    def get_products(self, rxn_id):
-        reaction = self.model.reactions.get_by_id(rxn_id)
-        return {k.id: v for k, v in reaction.metabolites.items() if v > 0}
-
     def get_exchange_reactions(self):
         rxns = [r.id for r in self.model.exchanges]
         return rxns
 
+    def get_gene_reactions(self):
+        """
+        :returns: A map of genes to reactions.
+        """
+        if not self._gene_to_reaction:
+            gr = dict()
+            for rxn_id in self.reactions:
+                rxn = self.model.reactions.get_by_id(rxn_id)
+                genes = rxn.genes
+                for g in genes:
+                    if g.id in gr.keys():
+                        gr[g.id].append(rxn_id)
+                    else:
+                        gr[g.id] = [rxn_id]
+            self._gene_to_reaction = gr
+        return self._gene_to_reaction
+
 
 class Simulation(CobraModelContainer, Simulator):
     """Generic Simulation class for cobra Model.
@@ -131,11 +138,11 @@ class Simulation(CobraModelContainer, Simulator):
 
         self.model = model
         self.model.solver = get_default_solver()
-        self.environmental_conditions = OrderedDict() if envcond is None else envcond
-        self.constraints = OrderedDict() if constraints is None else constraints
+        self._environmental_conditions = OrderedDict() if envcond is None else envcond
+        self._constraints = dict() if constraints is None else {
+            k: v for k, v in constraints.items() if k not in list(self._environmental_conditions.keys())}
+
         self.solver = solver
-        self._essential_reactions = None
-        self._essential_genes = None
         self._gene_to_reaction = None
         self._reference = reference
         self.__status_mapping = {
@@ -154,6 +161,32 @@ class Simulation(CobraModelContainer, Simulator):
         self._MAX_STR = 'maximize'
         self._MIN_STR = 'minimize'
 
+        # apply the env. cond. and additional constraints to the model
+        for r_id, bounds in self._environmental_conditions.items():
+            self._set_model_reaction_bounds(r_id, bounds)
+        for r_id, bounds in self._constraints.items():
+            self._set_model_reaction_bounds(r_id, bounds)
+
+    @property
+    def environmental_conditions(self):
+        return self._environmental_conditions.copy()
+
+    @environmental_conditions.setter
+    def environmental_conditions(self, a):
+        raise ValueError("Can not change environmental conditions. Use set_reaction_bounds instead")
+
+    def _set_model_reaction_bounds(self, r_id, bounds):
+        if isinstance(bounds, tuple):
+            lb = bounds[0]
+            ub = bounds[1]
+        elif isinstance(bounds, (int, float)):
+            lb = bounds
+            ub = bounds
+        else:
+            raise ValueError(f"Invalid bounds definition {bounds}")
+        rxn = self.model.reactions.get_by_id(r_id)
+        rxn.bounds = (lb, ub)
+
     @property
     def objective(self):
         from cobra.util.solver import linear_reaction_coefficients
@@ -173,83 +206,6 @@ class Simulation(CobraModelContainer, Simulator):
                 'The objective must be a reaction identifier or a dictionary of \
                 reaction identifier with respective coeficients.')
 
-    @property
-    def reference(self):
-        """The reference wild type reaction flux values.
-
-        :returns: A dictionary of wild type reaction flux values.
-
-        """
-        if self._reference is None:
-            self._reference = self.simulate(
-                method=SimulationMethod.pFBA).fluxes
-        return self._reference
-
-    def essential_reactions(self, min_growth=0.01):
-        """Essential reactions are those when knocked out enable a biomass flux value above a minimal growth defined as
-        a percentage of the wild type growth.
-
-        :param float min_growth: Minimal percentage of the wild type growth value. Default 0.01 (1%).
-        :returns: A list of essential reactions.
-
-        """
-        if self._essential_reactions is not None:
-            return self._essential_reactions
-        wt_solution = self.simulate()
-        wt_growth = wt_solution.objective_value
-        reactions = self.reactions
-        self._essential_reactions = []
-        for rxn in tqdm(reactions):
-            res = self.simulate(constraints={rxn: 0})
-            if res:
-                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) \
-                        or res.status == SStatus.INFEASIBLE:
-                    self._essential_reactions.append(rxn)
-        return self._essential_reactions
-
-    def essential_genes(self, min_growth=0.01):
-        """Essential genes are those when deleted enable a biomass flux value above a minimal growth defined as a
-        percentage of the wild type growth.
-
-        :param float min_growth: Minimal percentage of the wild type growth value. Default 0.01 (1%).
-        :returns: A list of essential genes.
-
-        """
-        if self._essential_genes is not None:
-            return self._essential_genes
-        self._essential_genes = []
-        wt_solution = self.simulate()
-        wt_growth = wt_solution.objective_value
-        genes = self.genes
-        for gene in tqdm(genes):
-            active_genes = set(self.genes) - {gene}
-            active_reactions = self.evaluate_gprs(active_genes)
-            inactive_reactions = set(self.reactions) - set(active_reactions)
-            gr_constraints = {rxn: 0 for rxn in inactive_reactions}
-            res = self.simulate(constraints=gr_constraints)
-            if res:
-                if (res.status == SStatus.OPTIMAL and res.objective_value < wt_growth * min_growth) \
-                        or res.status == SStatus.INFEASIBLE:
-                    self._essential_genes.append(gene)
-        return self._essential_genes
-
-    def evaluate_gprs(self, active_genes):
-        """Returns the list of active reactions for a given list of active genes.
-
-        :param list active_genes: List of genes identifiers.
-        :returns: A list of active reaction identifiers.
-
-        """
-        active_reactions = []
-        for r_id in self.reactions:
-            reaction = self.model.reactions.get_by_id(r_id)
-            if reaction.gene_reaction_rule:
-                if evaluate_expression_tree(str(reaction.gene_reaction_rule), active_genes):
-                    active_reactions.append(r_id)
-            else:
-                active_reactions.append(r_id)
-        return active_reactions
-
     def add_metabolite(self, id, formula=None, name=None, compartment=None):
         from cobra import Metabolite
         meta = Metabolite(id, formula=formula, name=name, compartment=compartment)
@@ -319,7 +275,7 @@ class Simulation(CobraModelContainer, Simulator):
         """Returns the list of genes that only catalyze transport reactions.
         """
         trp_rxs = self.get_transport_reactions()
-        r_g = self.gene_reactions()
+        r_g = self.get_gene_reactions()
         genes = []
         for g, rxs in r_g.items():
             if set(rxs).issubset(set(trp_rxs)):
@@ -344,23 +300,6 @@ class Simulation(CobraModelContainer, Simulator):
                 return rev
         return None
 
-    def gene_reactions(self):
-        """
-        :returns: A map of genes to reactions.
-        """
-        if not self._gene_to_reaction:
-            gr = OrderedDict()
-            for rxn_id in self.reactions:
-                rxn = self.model.reactions.get_by_id(rxn_id)
-                genes = rxn.genes
-                for g in genes:
-                    if g in gr.keys():
-                        gr[g.id].append(rxn_id)
-                    else:
-                        gr[g.id] = [rxn_id]
-            self._gene_to_reaction = gr
-        return self._gene_to_reaction
-
     def metabolite_reaction_lookup(self, force_recalculate=False):
         """ Return the network topology as a nested map from metabolite to reaction to coefficient.
         :return: a dictionary lookup table
@@ -375,7 +314,7 @@ class Simulation(CobraModelContainer, Simulator):
         return self._m_r_lookup
 
     def metabolite_elements(self, metabolite_id):
-        return self.model.metabolites.get_by_id(metabolite_id).elements()
+        return self.model.metabolites.get_by_id(metabolite_id).elements
 
     def get_reaction_bounds(self, reaction):
         """
@@ -385,13 +324,7 @@ class Simulation(CobraModelContainer, Simulator):
         :return: lb(s), ub(s), tuple
 
         """
-
-        if reaction in self.constraints:
-            lb, ub = self.constraints[reaction]
-        elif reaction in self.environmental_conditions:
-            lb, ub = self.environmental_conditions[reaction]
-        else:
-            lb, ub = self.model.reactions.get_by_id(reaction).bounds
+        lb, ub = self.model.reactions.get_by_id(reaction).bounds
         return lb if lb > -np.inf else ModelConstants.REACTION_LOWER_BOUND,\
             ub if ub < np.inf else ModelConstants.REACTION_UPPER_BOUND
 
@@ -402,7 +335,11 @@ class Simulation(CobraModelContainer, Simulator):
         :param float lb: lower bound 
         :param float ub: upper bound
         """
-        rxn = self.model.reactions.get_by_id(r)
+        if rxn in self.get_uptake_reactions():
+            self._environmental_conditions[rxn] = (lb, ub)
+        else:
+            self._constraints[rxn] = (lb, ub)
+        rxn = self.model.reactions.get_by_id(reaction)
         rxn.bounds = (lb, ub)
 
     def find_bounds(self):
@@ -433,7 +370,7 @@ class Simulation(CobraModelContainer, Simulator):
 
     # The simulator
     def simulate(self, objective=None, method=SimulationMethod.FBA, maximize=True,
-                 constraints=None, reference=None, scalefactor=None, solver=None):
+                 constraints=None, reference=None, scalefactor=None, solver=None, slim=False):
         '''
         Simulates a phenotype when applying a set constraints using the specified method.
 
@@ -451,13 +388,10 @@ class Simulation(CobraModelContainer, Simulator):
         elif isinstance(objective, dict) and len(objective) > 0:
             objective = next(iter(objective.keys()))
 
-        simul_constraints = OrderedDict()
+        simul_constraints = {}
         if constraints:
-            simul_constraints.update(constraints)
-        if self.constraints:
-            simul_constraints.update(self.constraints)
-        if self.environmental_conditions:
-            simul_constraints.update(self.environmental_conditions)
+            simul_constraints.update({k: v for k, v in constraints.items()
+                                      if k not in list(self._environmental_conditions.keys())})
 
         with self.model as model:
             model.objective = objective
@@ -474,7 +408,10 @@ class Simulation(CobraModelContainer, Simulator):
             # such is the case with pytfa.core.Model... need to find some workaround
             objective_sense = self._MAX_STR if maximize else self._MIN_STR
             if method == SimulationMethod.FBA:
-                solution = model.optimize(objective_sense=objective_sense)
+                if slim:
+                    solution = model.slim_optimize()
+                else:
+                    solution = model.optimize(objective_sense=objective_sense)
             elif method == SimulationMethod.pFBA:
                 # make fraction_of_optimum a configurable parameter?
                 solution = pfba(model)
@@ -494,15 +431,18 @@ class Simulation(CobraModelContainer, Simulator):
                 raise Exception(
                     "Unknown method to perform the simulation.")
 
-        status = self.__status_mapping[solution.status]
+        if slim:
+            return solution
 
-        result = SimulationResult(model, solution.objective_value, fluxes=solution.fluxes.to_dict(OrderedDict),
-                                  status=status, envcond=self.environmental_conditions,
-                                  model_constraints=self.constraints,
-                                  simul_constraints=constraints,
-                                  maximize=maximize,
-                                  method=method)
-        return result
+        else:
+            status = self.__status_mapping[solution.status]
+            result = SimulationResult(model, solution.objective_value, fluxes=solution.fluxes.to_dict(OrderedDict),
+                                      status=status, envcond=self.environmental_conditions,
+                                      model_constraints=self._constraints.copy(),
+                                      simul_constraints=constraints,
+                                      maximize=maximize,
+                                      method=method)
+            return result
 
     def FVA(self, obj_frac=0.9, reactions=None, constraints=None, loopless=False, internal=None, solver=None,
             format='dict'):
@@ -524,10 +464,9 @@ class Simulation(CobraModelContainer, Simulator):
         from cobra.flux_analysis.variability import flux_variability_analysis
 
         simul_constraints = {}
-        if self.environmental_conditions:
-            simul_constraints.update(self.environmental_conditions)
         if constraints:
-            simul_constraints.update(constraints)
+            simul_constraints.update({k: v for k, v in constraints.items()
+                                      if k not in list(self._environmental_conditions.keys())})
 
         with self.model as model:
 
